SUBROUTINE HORVISCUV2DPOM(C2U,C2V)
!
!           HORVISCUV2D computes contributions from the 
!           the horizontal viscosity terms of the depth integrated moment
!           equations:
!
!           d(AM2D*(d(UA*D)/dy))/dy + d(AM2D*(d(UA*D)/dx))/dx
!  and
!           d(AM2D*(d(VA*D)/dx))/dx + d(AM2D*(d(VA*D)/dy))/dy
!
!           The contributions are 
!           stored in the 2-D fields C2U and C2V before
!           exit
!
!           d(*) is the differential operator,
!           D is the dynamic depth,
!           AM2D is the vertically averaged horizontal eddy viscosity.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE STATE
IMPLICIT NONE
REAL C2U(IM,JM),C2V(IM,JM)
!
!   Local variables.
!
!   XFLUX,YFLUX are used to store the viscosity fluxes 
!                     in the two space directions
!   DAM is used to store averages of AM
!
! ..
! .. Local Arrays ..
REAL :: XFLUX(IM,JM), YFLUX(IM,JM)
!
REAL AREA,NETFLUX
REAL DAM,DEP
INTEGER I,J
!
C2U = 0.
C2V = 0.
AREA = DX*DY
!
! Compute UA * DU 
!
XFLUX = 0.
!
!$OMP PARALLEL DO PRIVATE(I,J) FIRSTPRIVATE(DX,DY)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (FSM(I,J).NE.0.) THEN
      XFLUX(I,J) = DY*2*D(I,J)*AM2D(I,J)*(UA(I+1,J)-UA(I,J))/DX
    ENDIF
  END DO
END DO
YFLUX = 0.
!
!$OMP PARALLEL DO PRIVATE(I,J,DAM,DEP) FIRSTPRIVATE(DX,DY)
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (DVM(I-1,J+1).NE.0. .AND. DVM(I,J+1).NE.0.) THEN
      DAM = .25*(AM2D(I,J)+AM2D(I-1,J)+AM2D(I,J+1)+AM2D(I-1,J+1))
      DEP = .25*(D(I,J)+D(I-1,J)+D(I,J+1)+D(I-1,J+1))
!     YFLUX(I,J) = DX*DAM*DELTAI(I,J+1)/DY
      YFLUX(I,J) = DX*DEP*DAM*((UA(I,J+1)-UA(I,J))/DY + &
     &      (VA(I,J+1)-VA(I-1,J+1))/DX )
    ENDIF
  END DO
END DO
!
!   Compute the net viscosity fluxes into each ocean cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,NETFLUX) FIRSTPRIVATE(AREA)
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DUM(I,J).GT.0.0) THEN
        NETFLUX = XFLUX(I,J) - XFLUX(I-1,J) + &
    &             YFLUX(I,J) - YFLUX(I,J-1) 
        C2U(I,J) =   NETFLUX/AREA
    ENDIF
  END DO
END DO
!
!   Then the momentum equation in V.
!
!   Compute VA * DV
!
XFLUX = 0.
!
!$OMP PARALLEL DO PRIVATE(I,J,DAM,DEP) FIRSTPRIVATE(DX,DY)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (DUM(I+1,J-1).NE.0. .AND. DUM(I+1,J).NE.0.) THEN
      DAM = .25*(AM2D(I,J)+AM2D(I,J-1)+AM2D(I+1,J)+AM2D(I+1,J-1))
      DEP = .25*(D(I,J)+D(I,J-1)+D(I+1,J)+D(I+1,J-1))
!     XFLUX(I,J) = DY*DAM*DELTAI(I+1,J)/DX
      XFLUX(I,J) = DY*DEP*DAM*((UA(I+1,J)-UA(I+1,J-1))/DY + &
     &      (VA(I+1,J)-VA(I,J))/DX)
    ENDIF
  END DO
END DO
!
!  Compute the viscosity fluxes for the V-equation in Y-direction.
!
YFLUX = 0.
!
!$OMP PARALLEL DO PRIVATE(I,J) FIRSTPRIVATE(DX,DY)
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (FSM(I,J).NE.0.) THEN
        YFLUX(I,J) = DX*D(I,J)*AM2D(I,J)*(VA(I,J+1)-VA(I,J))/DY
    ENDIF
  END DO
END DO
!
!   Compute the net viscosity fluxes into each ocean cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,NETFLUX) FIRSTPRIVATE(AREA)
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DVM(I,J).GT.0.0) THEN
        NETFLUX = XFLUX(I,J) - XFLUX(I-1,J) + &
    &             YFLUX(I,J) - YFLUX(I,J-1) 
        C2V(I,J) =   NETFLUX/AREA
    ENDIF
  END DO
END DO
!
RETURN
END
