SUBROUTINE HORVISCUVPOM(UBCLIN,VBCLIN,C3U,C3V)
!
!           HORVISCUV computes the following terms from the
!           Baroclinic parts of the momentum equations: 
!
!           d(AM*(d(UBCLIN*D)/dy))/dy + d(AM*(d(UBCLIN*D)/dx))/dx
!  and
!           d(AM*(d(VBCLIN*D)/dx))/dx + d(AM*(d(VBCLIN*D)/dy))/dy
!
!           d(*) is the differential operator,
!           D is the dynamic depth,
!           AM is the horizontal eddy viscosity,
!           The net-fluxes for each cell are stored in the 3-D arrays
!           C3U and C3V.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE STATE
IMPLICIT NONE
REAL UBCLIN(IM,JM,KB),VBCLIN(IM,JM,KB),C3U(IM,JM,KB),C3V(IM,JM,KB)
!
!   Local variables.
!
!   XFLUX,YFLUX are used to store the diffusive fluxes in
!                     the three space directions
! ..
! .. Local Arrays ..
REAL :: XFLUX(IM,JM,KB), YFLUX(IM,JM,KB)
!
REAL AREA,NETFLUX
REAL DAM,DEP
INTEGER I,J,K
!
C3U = 0.
C3V = 0.
AREA = DX*DY
!
!
!  First the diffusive fluxes for the U equation in X-direction.
!
XFLUX = 0.
!
!$OMP PARALLEL DO PRIVATE(I,J,K) FIRSTPRIVATE(DX,DY)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (FSM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        XFLUX(I,J,K) = DY*2*D(I,J)*AM(I,J,K)*(UBCLIN(I+1,J,K)-UBCLIN(I,J,K))/DX
      END DO
    ENDIF
  END DO
END DO
!
!  Compute the diffusive fluxes for the U-equation in Y-direction.
!
YFLUX = 0.
!
!$OMP PARALLEL DO PRIVATE(I,J,K,DAM,DEP) FIRSTPRIVATE(DX,DY) 
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (DVM(I-1,J+1).NE.0. .AND. DVM(I,J+1).NE.0.) THEN
      DO K = 1,KB-1
        DAM = .25*(AM(I,J,K)+AM(I-1,J,K)+AM(I,J+1,K)+AM(I-1,J+1,K))
        DEP = .25*(D(I,J)+D(I-1,J)+D(I,J+1)+D(I-1,J+1))
!        YFLUX(I,J,K) = DX*DAM*DELTAI(I,J+1,K)/DY
        YFLUX(I,J,K) = DX*DEP*DAM*((UBCLIN(I,J+1,K)-UBCLIN(I,J,K))/DY + &
     &      (VBCLIN(I,J+1,K)-VBCLIN(I-1,J+1,K))/DX )
     END DO
    ENDIF
 END DO
END DO
!
!   Take a step to timestep n+1 for the U-equation
!   by adding up the fluxes in and out of the cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,K,NETFLUX) FIRSTPRIVATE(AREA) 
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DUM(I,J).GT.0.0) THEN
      DO K = 1,KB-1
        NETFLUX = XFLUX(I,J,K) - XFLUX(I-1,J,K) + &
    &             YFLUX(I,J,K) - YFLUX(I,J-1,K) 
        C3U(I,J,K) =   NETFLUX/AREA
      END DO
    ENDIF
  END DO
END DO
!
!
!   Then the momentum equation in V.
!
!   First the diffusive fluxes in X-direction.
!
XFLUX = 0.
!
!  Compute differences in FIELDD in X-direction
!
!$OMP PARALLEL DO PRIVATE(I,J,K,DAM,DEP) FIRSTPRIVATE(DX,DY)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (DUM(I+1,J-1).NE.0. .AND. DUM(I+1,J).NE.0.) THEN
      DO K = 1,KB-1
        DAM = .25*(AM(I,J,K)+AM(I,J-1,K)+AM(I+1,J,K)+AM(I+1,J-1,K))
        DEP = .25*(D(I,J)+D(I,J-1)+D(I+1,J)+D(I+1,J-1))
!       XFLUX(I,J,K) = DY*DAM*DELTAI(I+1,J,K)/DX
        XFLUX(I,J,K) = DY*DEP*DAM*((UBCLIN(I+1,J,K)-UBCLIN(I+1,J-1,K))/DY + &
     &      (VBCLIN(I+1,J,K)-VBCLIN(I,J,K))/DX)
      END DO
    ENDIF
 END DO
END DO
!
!  Compute the diffusive fluxes for the V-equation in Y-direction.
!
YFLUX = 0.
!
!$OMP PARALLEL DO PRIVATE(I,J,K) FIRSTPRIVATE(DX,DY)
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (FSM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        YFLUX(I,J,K) = DX*2*D(I,J)*AM(I,J,K)*(VBCLIN(I,J+1,K)-VBCLIN(I,J,K))/DY
      END DO
    ENDIF
  END DO
END DO
!
!   Take a step to timestep n+1 for the V-equation by adding up the
!   fluxes in and out of the cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,K,NETFLUX) FIRSTPRIVATE(AREA)
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DVM(I,J).GT.0.0) THEN
      DO K = 1,KB-1
        NETFLUX = XFLUX(I,J,K) - XFLUX(I-1,J,K) + &
    &             YFLUX(I,J,K) - YFLUX(I,J-1,K) 
        C3V(I,J,K) =   NETFLUX/AREA
      END DO
    ENDIF
  END DO
END DO
!
!
RETURN
END
