SUBROUTINE LAXWUV
!
!           LAXWUV computes the following terms from
!           the moment equations on flux form:
!
!           - d(U*U*D)/d(X) - d(U*V*D)/d(Y) -  d(U*W)/d(SIGMA) 
!  and
!           - d(V*U*D)/d(X) - d(V*V*D)/d(Y) -  d(V*W)/d(SIGMA) 
!
!           from timestep n to timestep n+1
!           with a Space centered Lax-Wendroff scheme
!
!           d(*) is the differential operator,
!           D is the dynamic depth,
!           X and Y are the horisontal coordinates,
!           U is the horisontal velocity in X-direction,
!           V is the horisontal velocity in Y-direction,
!           SIGMA is the vertical coordinate,
!           W is the vertical velocity. 
!
! The contributions from these terms are added to the
! contributions from the internal pressure terms in the
! arrays DELTAU and DELTAV.
! This means that LAXWUV must be called after INTERNAL
! and before FBCOR.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE MOD_BOUND
IMPLICIT NONE
!
!   Local variables.
!
!   XFLUX,YFLUX,ZFLUX are used to store the advective fluxes and
!                     the final fluxes in
!                     the three space directions
!
! ..
! .. Local Arrays ..
REAL :: XFLUX(IM,JM,KB), YFLUX(IM,JM,KB), &
      ZFLUX(IM,JM,KB)
REAL :: FIELDD(IM,JM,KB),DELTAI(IM,JM,KB)
!
REAL SPEED
REAL AREA,NETFLUX,LAMBDAX,LAMBDAY
REAL AREATU,AREATV,AREATW
REAL UP,DAM
INTEGER I,J,K
!
LAMBDAX = DT/DX
LAMBDAY = DT/DY
!
!  River specific
!
IF (RIVLOG) THEN
  DO I = 1,NUMC
    IF (CODERIVER(I).EQ.1) THEN
      SPEED = FRIVER(I)/(DY*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I),JRIVER(I),K) = SPEED
        UADV(IRIVER(I),JRIVER(I),K) = SPEED
      END DO
    ELSEIF (CODERIVER(I).EQ.2) THEN
      SPEED = FRIVER(I)/(DX*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I),K) = SPEED
        VADV(IRIVER(I),JRIVER(I),K) = SPEED
      END DO
    ELSEIF (CODERIVER(I).EQ.3) THEN
      SPEED = FRIVER(I)/(DY*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I)+1,JRIVER(I),K) = -SPEED
        UADV(IRIVER(I)+1,JRIVER(I),K) = -SPEED
      END DO
    ELSEIF (CODERIVER(I).EQ.4) THEN
      SPEED = FRIVER(I)/(DX*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I)+1,K) = -SPEED
        VADV(IRIVER(I),JRIVER(I)+1,K) = -SPEED
      END DO
    ELSE
      WRITE(*,*)'ERROR IN SUPERBEEUV-1'
      STOP
    ENDIF
  END DO
ENDIF
!
!
!  First the advective and diffusive fluxes for the U equation in X-direction.
!
XFLUX = 0.
FIELDD = 0.
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      FIELDD(I,J,K) = U(I,J,K)*DU(I,J)
    END DO
  END DO
END DO
!
!  Compute differences in FIELDD in X-direction
!
DELTAI = 0.
DO I = 2,IM
  DO J = 1,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I-1,J,K)
    END DO
  END DO
END DO
!
!!$DOACROSS LOCAL(I,J,K,UP,AREATU)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (FSM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        UP = 0.5*(UADV(I,J,K)+UADV(I+1,J,K))
        AREATU = D(I,J)*UP
        XFLUX(I,J,K) = AREATU*UP - &
     &  0.5*LAMBDAX*UP*UP*DELTAI(I+1,J,K)
        XFLUX(I,J,K) = XFLUX(I,J,K)*DY
      END DO
    ENDIF
  END DO
END DO
!
!  Compute the advective and diffusive fluxes for the U-equation in Y-direction.
!
YFLUX = 0.
!
!  Compute differences in FIELDD in Y-direction
!
DELTAI = 0.
DO I = 1,IM
  DO J = 2,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I,J-1,K)
    END DO
  END DO
END DO
!!$DOACROSS LOCAL(I,J,K,UP,AREATV,DAM)
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (DVM(I-1,J+1).NE.0. .AND. DVM(I,J+1).NE.0.) THEN
      DO K = 1,KB-1
        UP = 0.5*(VADV(I-1,J+1,K)+VADV(I,J+1,K))
        AREATV = 0.5*(DV(I-1,J+1)+DV(I,J+1))*UP
        YFLUX(I,J,K) = 0.5*AREATV*(U(I,J,K)+U(I,J+1,K)) - &
     &  0.5*LAMBDAY*UP*UP*DELTAI(I,J+1,K)
        YFLUX(I,J,K) = YFLUX(I,J,K)*DX
     END DO
   ENDIF
 END DO
END DO
!
!  Compute the advective fluxes in sigma-direction for the U-equation.
!
ZFLUX = 0.
AREA = DX*DY
!!$DOACROSS LOCAL(I,J,K,UP,AREATW)
DO J = 2,JM-1
  DO I = 2,IM-1
    IF (DUM(I,J).NE.0.) THEN
      DO K = 2,KB-1
        UP = 0.5*(W(I,J,K)+W(I-1,J,K))
        AREATW = AREA*UP
        ZFLUX(I,J,K) = 0.5*AREATW*(U(I,J,K-1)+U(I,J,K)) - &
    &    0.5*AREA*DT*UP*UP*(U(I,J,K-1)-U(I,J,K))/(DZZ(K-1)*DU(I,J))
      END DO
    ENDIF
  END DO
END DO
!
!   Take a step to timestep n+1 for the U-equation
!   by adding up the fluxes in and out of the cell.
!
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DUM(I,J).GT.0.0) THEN
      DO K = 1,KB-1
        NETFLUX = XFLUX(I,J,K) - XFLUX(I-1,J,K) + &
    &             YFLUX(I,J,K) - YFLUX(I,J-1,K) + &
    &             (ZFLUX(I,J,K) - ZFLUX(I,J,K+1))/DZ(K)
        DELTAU(I,J,K) =  DELTAU(I,J,K) - NETFLUX/AREA
      END DO
    ENDIF
  END DO
END DO
!
!
!   Then the momentum equation in V.
!
!   First the sum of the advective and diffusive fluxes in X-direction.
!
XFLUX = 0.
FIELDD = 0.
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      FIELDD(I,J,K) = V(I,J,K)*DV(I,J)
    END DO
  END DO
END DO
!
!  Compute differences in FIELDD in X-direction
!
DELTAI = 0.
DO I = 2,IM
  DO J = 1,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I-1,J,K)
    END DO
  END DO
END DO
!!$DOACROSS LOCAL(I,J,K,UP,AREATU,DAM)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (DUM(I+1,J-1).NE.0. .AND. DUM(I+1,J).NE.0.) THEN
      DO K = 1,KB-1
        UP = 0.5*(UADV(I+1,J-1,K)+UADV(I+1,J,K))
        AREATU = 0.5*(DU(I+1,J-1)+DU(I+1,J))*UP
        XFLUX(I,J,K) = 0.5*AREATU*(V(I,J,K)+V(I+1,J,K)) - &
    & 0.5*LAMBDAX*UP*UP*DELTAI(I+1,J,K)
        XFLUX(I,J,K) = XFLUX(I,J,K)*DY
      END DO
    ENDIF
  END DO
END DO
!
!  Compute the advective and diffusive fluxes for the V-equation in Y-direction.
!
YFLUX = 0.
!
!  Compute differences in FIELDD in Y-direction
!
DELTAI = 0.
DO I = 1,IM
  DO J = 2,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I,J-1,K)
    END DO
  END DO
END DO
!!$DOACROSS LOCAL(I,J,K,UP,AREATV)
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (FSM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        UP = 0.5*(VADV(I,J,K)+VADV(I,J+1,K))
        AREATV = D(I,J)*UP
        YFLUX(I,J,K) = AREATV*UP - &
      & 0.5*LAMBDAY*UP*UP*DELTAI(I,J+1,K)
        YFLUX(I,J,K) = YFLUX(I,J,K)*DX
      END DO
    ENDIF
  END DO
END DO
!
!  Compute the advective fluxes for the V-equation in sigma-direction.
!
ZFLUX = 0.
AREA = DX*DY
!!$DOACROSS LOCAL(I,J,K,UP,AREATW)
DO J = 2,JM-1
  DO I = 2,IM-1
    IF (DVM(I,J).NE.0.) THEN
      DO K = 2,KB-1
        UP = 0.5*(W(I,J,K)+W(I,J-1,K))
        AREATW = AREA*UP
        ZFLUX(I,J,K) = 0.5*AREATW*(V(I,J,K-1)+V(I,J,K)) - &
    &    0.5*AREA*DT*UP*UP*(V(I,J,K-1)-V(I,J,K))/(DZZ(K-1)*DV(I,J))
      END DO
    ENDIF
  END DO
END DO
!
!   Take a step to timestep n+1 for the V-equation by adding up the
!   fluxes in and out of the cell.
!
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DVM(I,J).GT.0.0) THEN
      DO K = 1,KB-1
        NETFLUX = XFLUX(I,J,K) - XFLUX(I-1,J,K) + &
    &             YFLUX(I,J,K) - YFLUX(I,J-1,K) + &
    &             (ZFLUX(I,J,K) - ZFLUX(I,J,K+1))/DZ(K)
        DELTAV(I,J,K) =  DELTAV(I,J,K) - NETFLUX/AREA
      END DO
    ENDIF
  END DO
END DO
!
!
!  River specific
!
IF (RIVLOG) THEN
  DO I = 1,NUMC
    IF (CODERIVER(I).EQ.1) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I),JRIVER(I),K) = 0.
        UADV(IRIVER(I),JRIVER(I),K) = 0.
      END DO
    ELSEIF (CODERIVER(I).EQ.2) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I),K) = 0.
        VADV(IRIVER(I),JRIVER(I),K) = 0.
      END DO
    ELSEIF (CODERIVER(I).EQ.3) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I)+1,JRIVER(I),K) = 0.
        UADV(IRIVER(I)+1,JRIVER(I),K) = 0.
      END DO
    ELSEIF (CODERIVER(I).EQ.4) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I)+1,K) = 0.
        VADV(IRIVER(I),JRIVER(I)+1,K) = 0.
      END DO
    ELSE
      WRITE(*,*)'ERROR IN SUPERBEEUV-2'
      STOP
    ENDIF
  END DO
ENDIF
!
RETURN
END
