SUBROUTINE SUPERBEEUV
!
!           SUPERBEEUV computes the following terms from
!           the moment equations on flux form:
!
!           - d(U*U*D)/d(X) - d(U*V*D)/d(Y) -  d(U*W)/d(SIGMA)
!  and
!           - d(V*U*D)/d(X) - d(V*V*D)/d(Y) -  d(V*W)/d(SIGMA)
!
!           from timestep n to timestep n+1
!           with a superbee limiter TVD scheme. (Roe and Sweby)
!
!           d(*) is the differential operator,
!           D is the dynamic depth,
!           T is time,
!           X and Y are the horisontal coordinates,
!           U is the horisontal velocity in X-direction,
!           V is the horisontal velocity in Y-direction,
!           SIGMA is the vertical coordinate,
!           AM is the horizontal eddy viscosity,
!           W is the vertical velocity. 
! The contributions from these terms are added to the
! contributions from the internal pressure terms in the
! arrays DELTAU and DELTAV.
! This means that SUPERBEEUV must be called after INTERNAL
! and before MODESPLIT.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE MOD_BOUND
IMPLICIT NONE
!
!   Local variables.
!
!   XFLUXUP,YFLUXUP,ZFLUXUP are used to store upstream advective fluxes.
!   XFLUXLW,YFLUXLW,ZFLUXLW are used to store lax-wendroff advective fluxes.
!   XFLUX,YFLUX,ZFLUX are used to store the final fluxes in
!                     the three space directions
!
! ..
! .. Local Arrays ..
REAL :: XFLUX(IM,JM,KB), XFLUXLW(IM,JM,KB), XFLUXUP(IM,JM,KB), &
      YFLUX(IM,JM,KB), YFLUXLW(IM,JM,KB), YFLUXUP(IM,JM,KB), &
      ZFLUX(IM,JM,KB), ZFLUXLW(IM,JM,KB), ZFLUXUP(IM,JM,KB), &
      FIELDD(IM,JM,KB), DELTAI(IM,JM,KB)
!
REAL AREA,NETFLUX,VOLUME,VOLUMEP,LAMBDAX,LAMBDAY
REAL R,PHIR,NUMER,DENUM,EPSILON,AREATU,AREATV,AREATW
REAL DEPTHZ,UP,UPSIG
INTEGER I,J,K,NSIG
REAL SPEED
!
LAMBDAX = DT/DX
LAMBDAY = DT/DY
EPSILON = 1.E-10
!
!
!  First the advective and diffusive fluxes for the U equation in X-direction.
!
!  River specific
!
IF (RIVLOG) THEN
  DO I = 1,NUMC
    IF (CODERIVER(I).EQ.1) THEN
      SPEED = FRIVER(I)/(DY*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I),JRIVER(I),K) = SPEED
        UADV(IRIVER(I),JRIVER(I),K) = SPEED
      END DO
    ELSEIF (CODERIVER(I).EQ.2) THEN
      SPEED = FRIVER(I)/(DX*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I),K) = SPEED
        VADV(IRIVER(I),JRIVER(I),K) = SPEED
      END DO
    ELSEIF (CODERIVER(I).EQ.3) THEN
      SPEED = FRIVER(I)/(DY*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I)+1,JRIVER(I),K) = -SPEED
        UADV(IRIVER(I)+1,JRIVER(I),K) = -SPEED
      END DO
    ELSEIF (CODERIVER(I).EQ.4) THEN
      SPEED = FRIVER(I)/(DX*DZRIVER(I)*H(IRIVER(I),JRIVER(I)))
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I)+1,K) = -SPEED
        VADV(IRIVER(I),JRIVER(I)+1,K) = -SPEED
      END DO
    ELSE
      WRITE(*,*)'ERROR IN SUPERBEEUV-1'
      STOP
    ENDIF
  END DO
ENDIF 
XFLUX = 0.
XFLUXUP = 0.
XFLUXLW = 0.
FIELDD = 0.
!$OMP PARALLEL DO 
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      FIELDD(I,J,K) = U(I,J,K)*DU(I,J)
    END DO
  END DO
END DO
!
!  Compute differences in FIELDD in X-direction
!
DELTAI = 0.
!$OMP PARALLEL DO 
DO I = 2,IM
  DO J = 1,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I-1,J,K)
    END DO
  END DO
END DO

!$OMP PARALLEL DO PRIVATE(UP,AREATU,UPSIG,NSIG,NUMER,DENUM,R,PHIR)  FIRSTPRIVATE(LAMBDAX,DY) 
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (FSM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        UP = 0.5*(UADV(I,J,K)+UADV(I+1,J,K))
        AREATU = D(I,J)*UP
        IF (UP.GE.0.) THEN
          XFLUXUP(I,J,K) = AREATU*U(I,J,K)
          IF (I.GE.2) THEN
            UPSIG = 0.5*(UADV(I-1,J,K)+UADV(I,J,K))
            NSIG = 1
          ENDIF
        ELSE
          XFLUXUP(I,J,K) = AREATU*U(I+1,J,K)
          IF (I.LE.IM-2) THEN
            UPSIG = 0.5*(UADV(I+1,J,K)+UADV(I+2,J,K))
            NSIG = -1
          ENDIF
        ENDIF
        XFLUXLW(I,J,K) = AREATU*UP - &
      & 0.5*LAMBDAX*UP*UP*DELTAI(I+1,J,K)
        IF (I.GE.2 .AND. I.LE.IM-2) THEN
          NUMER = (ABS(UPSIG) - LAMBDAX*UPSIG**2)*  &
      &         (D(I-NSIG,J)*(U(I+1-NSIG,J,K)-U(I-NSIG,J,K)))
          DENUM = (ABS(UP) - LAMBDAX*UP**2)*  &
      &         (D(I,J)*(U(I+1,J,K)-U(I,J,K)))
          IF (ABS(DENUM).GT.EPSILON) THEN
            R = NUMER/DENUM
            PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
          ELSE
            PHIR = 0.
          ENDIF
        ELSE
          PHIR = 0.
        ENDIF
        XFLUX(I,J,K) = XFLUXUP(I,J,K) + PHIR*(XFLUXLW(I,J,K)-XFLUXUP(I,J,K))
        XFLUX(I,J,K) = XFLUX(I,J,K)*DY
      END DO
    ENDIF
  END DO
END DO
!
!  Compute the advective and diffusive fluxes for the U-equation in Y-direction.
!
YFLUX = 0.
YFLUXUP = 0.
YFLUXLW = 0.
!
!  Compute differences in FIELDD in Y-direction
!
DELTAI = 0.

!$OMP PARALLEL DO 
DO I = 1,IM
  DO J = 2,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I,J-1,K)
    END DO
  END DO
END DO
!$OMP PARALLEL DO PRIVATE(UP,AREATV,UPSIG,NSIG,NUMER,DENUM,R,PHIR)  FIRSTPRIVATE(LAMBDAY,DX) 
DO K = 1,KB-1
  DO J = 1,JM-1
    DO I = 2,IM-1
      IF (DVM(I-1,J+1).NE.0. .AND. DVM(I,J+1).NE.0.) THEN
        UP = 0.5*(VADV(I-1,J+1,K)+VADV(I,J+1,K))
        AREATV = 0.5*(DV(I-1,J+1)+DV(I,J+1))*UP
        IF (UP.GE.0.) THEN
          YFLUXUP(I,J,K) = AREATV*U(I,J,K)
          UPSIG = 0.5*(VADV(I-1,J,K)+VADV(I,J,K))
          NSIG = 1
        ELSE
          YFLUXUP(I,J,K) = AREATV*U(I,J+1,K)
          IF (J.LE.JM-2) THEN
            UPSIG = 0.5*(VADV(I-1,J+2,K)+VADV(I,J+2,K))
            NSIG = -1
          ENDIF
        ENDIF
        YFLUXLW(I,J,K) = 0.5*AREATV*(U(I,J,K)+U(I,J+1,K)) - &
    & 0.5*LAMBDAY*UP*UP*DELTAI(I,J+1,K)
        IF (J.GE.2 .AND. J.LE.JM-2) THEN
          NUMER = (ABS(UPSIG) - LAMBDAY*UPSIG**2)*  &
    &         0.5*(DV(I-1,J+1-NSIG)+DV(I,J+1-NSIG))* &
    &         (U(I,J+1-NSIG,K) - U(I,J-NSIG,K))
          DENUM = (ABS(UP) - LAMBDAY*UP**2)*  &
    &         0.5*(DV(I-1,J+1)+DV(I,J+1))*(U(I,J+1,K)-U(I,J,K))
          IF (ABS(DENUM).GT.EPSILON) THEN
            R = NUMER/DENUM
            PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
          ELSE
            PHIR = 0.
          ENDIF
        ELSE
          PHIR = 0.
        ENDIF
        YFLUX(I,J,K) = YFLUXUP(I,J,K) + PHIR*(YFLUXLW(I,J,K)-YFLUXUP(I,J,K))
        YFLUX(I,J,K) = YFLUX(I,J,K)*DX
     ENDIF
   END DO
 END DO
END DO
!
!  Compute the advective fluxes in sigma-direction for the U-equation.
!
ZFLUX = 0.
ZFLUXUP = 0.
ZFLUXLW = 0.
AREA = DX*DY
!!$OMP PARALLEL DO PRIVATE(UP,AREATW,UPSIG,NSIG,NUMER,DENUM,R,PHIR)  FIRSTPRIVATE(DZZ,AREA,DT)
DO J = 2,JM-1
  DO I = 2,IM-1
    IF (DUM(I,J).NE.0.) THEN
      DO K = 2,KB-1
        UP = 0.5*(W(I,J,K)+W(I-1,J,K))
        AREATW = AREA*UP
        IF (UP.GE.0.) THEN
          ZFLUXUP(I,J,K) = AREATW*U(I,J,K)
          UPSIG =  0.5*(W(I,J,K+1)+W(I-1,J,K+1))
          NSIG = -1
        ELSE
          ZFLUXUP(I,J,K) = AREATW*U(I,J,K-1)
          UPSIG =  0.5*(W(I,J,K-1)+W(I-1,J,K-1))
          NSIG = 1
        ENDIF
        ZFLUXLW(I,J,K) = 0.5*AREATW*(U(I,J,K-1)+U(I,J,K)) - &
    &    0.5*AREA*DT*UP*UP*(U(I,J,K-1)-U(I,J,K))/(DZZ(K-1)*DU(I,J))
      IF ( (K.EQ.2 .AND. NSIG.EQ.-1) .OR. &
    &      (K.GE.3 .AND. K.LE.KB-1)) THEN
        NUMER = (ABS(UPSIG) - (DT/(DZZ(K-1-NSIG)*DU(I,J)))* &
    &         UPSIG**2)*  &
    &         (U(I,J,K-1-NSIG)-U(I,J,K-NSIG))
        DENUM = (ABS(UP) - (DT/(DZZ(K-1)*DU(I,J)))* &
    &         UP**2)*  &
    &         (U(I,J,K-1)-U(I,J,K))
        IF (ABS(DENUM).GT.EPSILON) THEN
          R = NUMER/DENUM
          PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
        ELSE
          PHIR = 0.
        ENDIF
      ELSE
        PHIR = 1.
      ENDIF
!
        ZFLUX(I,J,K) = ZFLUXUP(I,J,K) + PHIR*(ZFLUXLW(I,J,K)-ZFLUXUP(I,J,K))
      END DO
    ENDIF
  END DO
END DO
!
!   Take a step to timestep n+1 for the U-equation
!   by adding up the fluxes in and out of the cell.
!
!$OMP PARALLEL DO PRIVATE(NETFLUX) 
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DUM(I,J).GT.0.0) THEN
      DO K = 1,KB-1
        NETFLUX = XFLUX(I,J,K) - XFLUX(I-1,J,K) + &
    &             YFLUX(I,J,K) - YFLUX(I,J-1,K) + &
    &             (ZFLUX(I,J,K) - ZFLUX(I,J,K+1))/DZ(K)
        DELTAU(I,J,K) =  DELTAU(I,J,K) - NETFLUX/AREA
      END DO
    ENDIF
  END DO
END DO
!
!
!   Then the momentum equation in V.
!
!   First the sum of the advective and diffusive fluxes in X-direction.
!
XFLUX = 0.
XFLUXUP = 0.
XFLUXLW = 0.
FIELDD = 0.
!$OMP PARALLEL DO 
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      FIELDD(I,J,K) = V(I,J,K)*DV(I,J)
    END DO
  END DO
END DO
!
!  Compute differences in FIELDD in X-direction
!
DELTAI = 0.
!$OMP PARALLEL DO 
DO I = 2,IM
  DO J = 1,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I-1,J,K)
    END DO
  END DO
END DO
!$OMP PARALLEL DO PRIVATE(UP,AREATU,UPSIG,NSIG,NUMER,DENUM,R,PHIR) FIRSTPRIVATE(LAMBDAX,DY) 
DO K = 1,KB-1
  DO J = 2,JM-1
    DO I = 1,IM-1
      IF (DUM(I+1,J-1).NE.0. .AND. DUM(I+1,J).NE.0.) THEN
        UP = 0.5*(UADV(I+1,J-1,K)+UADV(I+1,J,K))
        AREATU = 0.5*(DU(I+1,J-1)+DU(I+1,J))*UP
        IF (UP.GE.0.) THEN
          XFLUXUP(I,J,K) = AREATU*V(I,J,K)
          UPSIG = 0.5*(UADV(I,J-1,K)+UADV(I,J,K))
          NSIG = 1
        ELSE
          XFLUXUP(I,J,K) = AREATU*V(I+1,J,K)
          IF (I.LE.IM-2) THEN
            UPSIG = 0.5*(UADV(I+2,J-1,K)+UADV(I+2,J,K))
            NSIG = -1
          ENDIF
        ENDIF
        XFLUXLW(I,J,K) = 0.5*AREATU*(V(I,J,K)+V(I+1,J,K)) - &
    &   0.5*LAMBDAX*UP*UP*DELTAI(I+1,J,K)
        IF (I.GE.2 .AND. I.LE.IM-2) THEN
          NUMER = (ABS(UPSIG) - LAMBDAX*UPSIG**2)*  &
    &         0.5*(DU(I+1-NSIG,J-1)+DU(I+1-NSIG,J))* &
    &         (V(I+1-NSIG,J,K) - V(I-NSIG,J,K))
          DENUM = (ABS(UP) - LAMBDAX*UP**2)*  &
    &         0.5*(DU(I+1,J-1)+DU(I+1,J))*(V(I+1,J,K)-V(I,J,K))
          IF (ABS(DENUM).GT.EPSILON) THEN
            R = NUMER/DENUM
            PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
          ELSE
            PHIR = 0.
          ENDIF
        ELSE
          PHIR = 0.
        ENDIF
        XFLUX(I,J,K) = XFLUXUP(I,J,K) + PHIR*(XFLUXLW(I,J,K)-XFLUXUP(I,J,K))
        XFLUX(I,J,K) = XFLUX(I,J,K)*DY
      ENDIF
   END DO
 END DO
END DO
!
!  Compute the advective and diffusive fluxes for the V-equation in Y-direction.
!
YFLUX = 0.
YFLUXUP = 0.
YFLUXLW = 0.
!
!  Compute differences in FIELDD in Y-direction
!
DELTAI = 0.
!$OMP PARALLEL DO 
DO I = 1,IM
  DO J = 2,JM
    DO K = 1,KB
      DELTAI(I,J,K) = FIELDD(I,J,K)-FIELDD(I,J-1,K)
    END DO
  END DO
END DO
!$OMP PARALLEL DO PRIVATE(UP,AREATV,UPSIG,NSIG,NUMER,DENUM,R,PHIR) FIRSTPRIVATE(LAMBDAY,DX) 
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (FSM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        UP = 0.5*(VADV(I,J,K)+VADV(I,J+1,K))
        AREATV = D(I,J)*UP
        IF (UP.GE.0.) THEN
          YFLUXUP(I,J,K) = AREATV*V(I,J,K)
          IF (J.GE.2) THEN
            UPSIG = 0.5*(VADV(I,J-1,K)+VADV(I,J,K))
            NSIG = 1
          ENDIF
        ELSE
          YFLUXUP(I,J,K) = AREATV*V(I,J+1,K)
          IF (J.LE.JM-2) THEN
            UPSIG = 0.5*(VADV(I,J+1,K)+VADV(I,J+2,K))
            NSIG = -1
          ENDIF
        ENDIF
        YFLUXLW(I,J,K) = AREATV*UP - &
      & 0.5*LAMBDAY*UP*UP*DELTAI(I,J+1,K)
        IF (J.GE.2 .AND. J.LE.JM-2) THEN
          NUMER = (ABS(UPSIG) - LAMBDAY*UPSIG**2)*  &
      &         (D(I,J-NSIG)*(V(I,J+1-NSIG,K)-V(I,J-NSIG,K)))
          DENUM = (ABS(UP) - LAMBDAY*UP**2)*  &
      &         (D(I,J)*(V(I,J+1,K)-V(I,J,K)))
          IF (ABS(DENUM).GT.EPSILON) THEN
            R = NUMER/DENUM
            PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
          ELSE
            PHIR = 0.
          ENDIF
        ELSE
          PHIR = 0.
        ENDIF
        YFLUX(I,J,K) = YFLUXUP(I,J,K) + PHIR*(YFLUXLW(I,J,K)-YFLUXUP(I,J,K))
        YFLUX(I,J,K) = YFLUX(I,J,K)*DX
      END DO
    ENDIF
  END DO
END DO
!
!  Compute the advective fluxes for the V-equation in sigma-direction.
!
ZFLUX = 0.
ZFLUXUP = 0.
ZFLUXLW = 0.
AREA = DX*DY
!!$OMP PARALLEL DO PRIVATE(UP,AREATW,UPSIG,NSIG,NUMER,DENUM,R,PHIR) FIRSTPRIVATE(AREA,DZZ,DT)
DO J = 2,JM-1
  DO I = 2,IM-1
    IF (DVM(I,J).NE.0.) THEN
      DO K = 2,KB-1
        UP = 0.5*(W(I,J,K)+W(I,J-1,K))
        AREATW = AREA*UP
        IF (UP.GE.0.) THEN
          ZFLUXUP(I,J,K) = AREATW*V(I,J,K)
          UPSIG =  0.5*(W(I,J,K+1)+W(I,J-1,K+1))
          NSIG = -1
        ELSE
          ZFLUXUP(I,J,K) = AREATW*V(I,J,K-1)
          UPSIG =  0.5*(W(I,J,K-1)+W(I,J-1,K-1))
          NSIG = 1
        ENDIF
        ZFLUXLW(I,J,K) = 0.5*AREATW*(V(I,J,K-1)+V(I,J,K)) - &
    &    0.5*AREA*DT*UP*UP*(V(I,J,K-1)-V(I,J,K))/(DZZ(K-1)*DV(I,J))
      IF ( (K.EQ.2 .AND. NSIG.EQ.-1) .OR. &
    &      (K.GE.3 .AND. K.LE.KB-1)) THEN
        NUMER = (ABS(UPSIG) - (DT/(DZZ(K-1-NSIG)*DV(I,J)))* &
    &         UPSIG**2)*  &
    &         (V(I,J,K-1-NSIG)-V(I,J,K-NSIG))
        DENUM = (ABS(UP) - (DT/(DZZ(K-1)*DV(I,J)))* &
    &         UP**2)*  &
    &         (V(I,J,K-1)-V(I,J,K))
        IF (ABS(DENUM).GT.EPSILON) THEN
          R = NUMER/DENUM
          PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
        ELSE
          PHIR = 0.
        ENDIF
      ELSE
        PHIR = 1.
      ENDIF
!
        ZFLUX(I,J,K) = ZFLUXUP(I,J,K) + PHIR*(ZFLUXLW(I,J,K)-ZFLUXUP(I,J,K))
      END DO
    ENDIF
  END DO
END DO
!
!   Take a step to timestep n+1 for the V-equation by adding up the
!   fluxes in and out of the cell.
!
!$OMP PARALLEL DO PRIVATE(NETFLUX)
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DVM(I,J).GT.0.0) THEN
      DO K = 1,KB-1
        NETFLUX = XFLUX(I,J,K) - XFLUX(I-1,J,K) + &
    &             YFLUX(I,J,K) - YFLUX(I,J-1,K) + &
    &             (ZFLUX(I,J,K) - ZFLUX(I,J,K+1))/DZ(K)
        DELTAV(I,J,K) =  DELTAV(I,J,K) - NETFLUX/AREA
      END DO
    ENDIF
  END DO
END DO
!
!
!  River specific
!
IF (RIVLOG) THEN
  DO I = 1,NUMC
    IF (CODERIVER(I).EQ.1) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I),JRIVER(I),K) = 0.
        UADV(IRIVER(I),JRIVER(I),K) = 0.
      END DO
    ELSEIF (CODERIVER(I).EQ.2) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I),K) = 0.
        VADV(IRIVER(I),JRIVER(I),K) = 0.
      END DO
    ELSEIF (CODERIVER(I).EQ.3) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        U(IRIVER(I)+1,JRIVER(I),K) = 0.
        UADV(IRIVER(I)+1,JRIVER(I),K) = 0.
      END DO
    ELSEIF (CODERIVER(I).EQ.4) THEN
      DO K = KRIVER1(I),KRIVER2(I)
        V(IRIVER(I),JRIVER(I)+1,K) = 0.
        VADV(IRIVER(I),JRIVER(I)+1,K) = 0.
      END DO
    ELSE
      WRITE(*,*)'ERROR IN SUPERBEEUV-2'
      STOP
    ENDIF
  END DO
ENDIF 
RETURN
END
