! 2000-06-27 HA Changed 3D to 1D arrays for the tridiagonal solver,
! made them private in the OpenMP directive.
!
! 2000-07-12 HA Made loops go over 1,pt, fetching i,j from
!  appropriate arrays. easier for compiler to optimize(avoid if stm.
!  in loops) - and when automatically parallelizing we get load
!  balancing of ocean cells! 
!
SUBROUTINE VERTVISCUVB(UBCLIN,VBCLIN)
!
!  VERTVISCUV propagates the solution of the equations
!
!  d(UD)/dt = d((KM/D)*d(U)/d(sigma))/d(sigma)
!
!  d(VD)/dt = d((KM/D)*d(V)/d(sigma))/d(sigma)
!
!  with boundary conditions:
!
!  (KM/D)*d(U)/d(sigma) = WSURFX  at sigma=0
!  (KM/D)*d(V)/d(sigma) = WSURFY  at sigma=0
!  (KM/D)*d(U)/d(sigma) = CBC*ABS((U,V))*U at sigma=-1
!  (KM/D)*d(V)/d(sigma) = CBC*ABS((U,V))*V at sigma=-1
!
!  where (WSURFX,WSURFY) are the surface wind stresses in X and Y directions
!  and D the bottom drag coefficient.
!
!  The solutions are propagated with an fully implicit method.
!
USE STATE
IMPLICIT NONE
!
! Local variables.
!
INTEGER I,J,K,pt
REAL DUSQ,FRICT,DVSQ
REAL, DIMENSION(KB-1)  :: RHS,A,B,C,XX,ALF,BET,G
REAL, DIMENSION(IM,JM,KB) :: KMU,KMV,UBCLIN,VBCLIN
!
!  First propagate the U-equation over all internal sea points.
!
!  Average vertical viscosity coefficients to U-points.
!

!$OMP PARALLEL DO PRIVATE(i,j,k)
DO j=2,jm-1
   DO k=2,kb   
      DO i=2,im-1
         KMU(i,j,k) = 0.5*(KM(i-1,j,k)+KM(i,j,k))
         KMV(i,j,k) = 0.5*(KM(i,j-1,k)+KM(i,j,k))
      END DO
   END DO
END DO


!$OMP PARALLEL DO PRIVATE(I,J,K,DUSQ,FRICT,RHS,A,B,C,XX,ALF,BET,G) &
!$omp  firstprivate(dt,dz,dzz)
DO pt=1,numup
   i=indu(1,pt)
   j=indu(2,pt)

!DO J = 2,JM-1
!  DO I = 2,IM
!    IF (DUM(I,J).NE.0.) THEN
      DUSQ = DU(I,J)*DU(I,J)
      
      FRICT = 0.5*(CBC(I,J)+CBC(I-1,J))*SQRT(U(I,J,KB-1)**2 &
           + (0.25*(V(I,J,KB-1)  &
           + V(I,J+1,KB-1)+V(I-1,J,KB-1)+V(I-1,J+1,KB-1)))**2)
      
!
! Define the right hand side of the solution matrix.
!
      RHS(1) = Ubclin(I,J,1) + DT*WUSURF(I,J)/(DU(I,J)*DZ(1))
      DO K = 2,KB-2
        RHS(K) = Ubclin(I,J,K)
      END DO
      RHS(KB-1) = UBCLIN(I,J,KB-1) - DT*FRICT*UA(I,J)/(DU(I,J)*DZ(KB-1))
      
! Define the upper diagonal elements of the tridiagonal solution matrix.
!
      DO K = 1,KB-2
        B(K) = -DT*KMU(I,J,K+1)/(DUSQ*DZ(K)*DZZ(K))
      END DO
!
! Define the lower diagonal elements.
!
      DO K = 2,KB-1
        C(K) = -DT*KMU(I,J,K)/(DUSQ*DZ(K)*DZZ(K-1))
      END DO
!
! Compute bottom friction.
!
!
! Define the diagonal elements.
!
      A(1) = 1. - B(1)
      DO K = 2,KB-2
        A(K) = 1. - B(K) - C(K)
      END DO
      A(KB-1) = 1. - C(KB-1) + DT*FRICT/(DU(I,J)*DZ(KB-1))
!
! Solve the system of equations.
!
!
!   LU-decompose the linear system.
!
      ALF(1) = A(1)
      DO K = 2,KB-1
        ALF(K) = A(K) - C(K)*B(K-1)/ALF(K-1)
      END DO
      DO K = 2,KB-1
        BET(K) = C(K)/ALF(K-1)
      END DO
!
!   Forward substitute the right hand side.
!
      G(1) = RHS(1)
      DO K = 2,KB-1
        G(K) = RHS(K) - BET(K)*G(K-1)
      END DO
!
!   Back substitute to compute the solution U.
!
      Ubclin(I,J,KB-1) = G(KB-1)/ALF(KB-1)
      DO K = KB-2,1,-1
        Ubclin(I,J,K) = (G(K) - B(K)*Ubclin(I,J,K+1))/ALF(K)
      END DO
      WUBOT(I,J) = -FRICT*(Ubclin(I,J,KB-1)+ua(i,j))
!    ENDIF
!  END DO
END DO
!
!  Then propagate the V-equation over all internal sea points.
!
!
!$omp parallel do private(i,j,k,dvsq,rhs,a,b,c,xx,alf,bet,g,frict) &
!$omp  firstprivate(dt,dz,dzz)
DO pt=1,numvp
   i=indv(1,pt)
   j=indv(2,pt)
!DO J = 2,JM-1
!  DO I = 2,IM
!    IF (DVM(I,J).NE.0.) THEN

      DVSQ = DV(I,J)*DV(I,J)
!
! Compute bottom friction.
!
   FRICT = 0.5*(CBC(I,J)+CBC(I,J-1))*SQRT(V(I,J,KB-1)**2 + (0.25*(U(I,J,KB-1)+&
      &  U(I+1,J,KB-1)+U(I,J-1,KB-1)+U(I+1,J-1,KB-1)))**2)
      
!
! Define the right hand side of the solution matrix.
!
      RHS(1) = Vbclin(I,J,1) + DT*WVSURF(I,J)/(DV(I,J)*DZ(1))
      DO K = 2,KB-2
        RHS(K) = Vbclin(I,J,K)
      END DO
      RHS(KB-1) = VBCLIN(I,J,KB-1) - DT*FRICT*VA(I,J)/(DV(I,J)*DZ(KB-1))!
! Define the upper diagonal elements of the tridiagonal solution matrix.
!
      DO K = 1,KB-2
        B(K) = -DT*KMV(I,J,K+1)/(DVSQ*DZ(K)*DZZ(K))
      END DO
!
! Define the lower diagonal elements.
!
      DO K = 2,KB-1
        C(K) = -DT*KMV(I,J,K)/(DVSQ*DZ(K)*DZZ(K-1))
      END DO
!
! Define the diagonal elements.
!
      A(1) = 1. - B(1)
      DO K = 2,KB-2
        A(K) = 1. - B(K) - C(K)
      END DO
      A(KB-1) = 1. - C(KB-1) + DT*FRICT/(DV(I,J)*DZ(KB-1))
!
! Solve the system of equations.
!
!
!   LU-decompose the linear system.
!
      ALF(1) = A(1)
      DO K = 2,KB-1
        ALF(K) = A(K) - C(K)*B(K-1)/ALF(K-1)
      END DO
      DO K = 2,KB-1
        BET(K) = C(K)/ALF(K-1)
      END DO
!
!   Forward substitute the right hand side.
!
      G(1) = RHS(1)
      DO K = 2,KB-1
        G(K) = RHS(K) - BET(K)*G(K-1)
      END DO
!
!   Back substitute to compute the solution V.
!
      Vbclin(I,J,KB-1) = G(KB-1)/ALF(KB-1)
      DO K = KB-2,1,-1
        Vbclin(I,J,K) = (G(K) - B(K)*Vbclin(I,J,K+1))/ALF(K)
      END DO
      WVBOT(I,J) = -FRICT*(Vbclin(I,J,KB-1)+va(i,j))
!    ENDIF
!  END DO
END DO
!
RETURN
END
